# coding=utf-8

__all__ = ['ASYNC_EXECUTOR', 'ASYNC_ANNOTATION_PROCESSOR',
           'TASK_SCHEDULER', 'SCHEDULED_ANNOTATION_PROCESSOR']

ASYNC_EXECUTOR = '__async_executor'
ASYNC_ANNOTATION_PROCESSOR = '__async_annotation_processor'

TASK_SCHEDULER = '__task_scheduler'
SCHEDULED_ANNOTATION_PROCESSOR = '__scheduled_annotation_processor'
