<!DOCTYPE html>
<html lang="en">
<head>
    <meta charset="UTF-8">
    <meta name="viewport" content="width=device-width, initial-scale=1, shrink-to-fit=no">
    <title>Welcome to guniflask!</title>
    <style>
        body {
            width: 35em;
            margin: 0 auto;
            font-family: Tahoma, Verdana, Arial, sans-serif;
            color: #333;
        }
    </style>
</head>
<body>
<h1>Welcome to guniflask!</h1>
<p>If you see this page, the guniflask is successfully installed and working.</p>
<p>For a guide and recipes on how to configure / customize this project, check out the
    <a href="https://guniflask.readthedocs.io/">guniflask documentation</a>.</p>
<p><em>Thank you for using guniflask.</em></p>
</body>
</html>
