# coding=utf-8

__all__ = ['APPLICATION_EVENT_PUBLISHER',
           'CONFIGURATION_ANNOTATION_PROCESSOR',
           'CONFIGURATION_BEAN_NAME_GENERATOR']

APPLICATION_EVENT_PUBLISHER = '__application_event_publisher'

CONFIGURATION_ANNOTATION_PROCESSOR = '__configuration_annotation_processor'

CONFIGURATION_BEAN_NAME_GENERATOR = '__configuration_bean_name_generator'
